==> fred-reporter-0.1.0.tar.gz: fred-reporter-0.1.0/reporter/report.py <==
"""Report related data structures."""
import json
from typing import Any, Dict, Iterable, Iterator, List, Sequence


class AmbiguousColumnNames(Exception):
    """Error thrown when report result doesn't have unique column names."""


class Param:
    """DB report parameter."""

    def __init__(self, name: str, type: str):
        self.name = name
        self.type = type

    def to_dict(self):
        """Return param dict."""
        return {'name': self.name, 'type': self.type}

    def __repr__(self):
        return "Param<{}: {}>".format(self.name, self.type)

    def __eq__(self, other):
        return isinstance(other, Param) and self.name == other.name and self.type == other.type


class Report:
    """DB report."""

    def __init__(self, name: str, input_parameters: Sequence[Param], output_parameters: Sequence[Param]):
        self.name = name
        self.input_parameters = input_parameters
        self.output_parameters = output_parameters

    def to_dict(self):
        """Return report dict."""
        return {
            'name': self.name,
            'input_parameters': [param.to_dict() for param in self.input_parameters],
            'output_parameters': [param.to_dict() for param in self.output_parameters],
        }

    @property
    def is_unambiguous(self) -> bool:
        """Return whether all input param names are distinct and all output param names are distinct."""
        input_names = [param.name for param in self.input_parameters]
        output_names = [param.name for param in self.output_parameters]
        return len(input_names) == len(set(input_names)) and len(output_names) == len(set(output_names))

    def __eq__(self, other):
        return (
            isinstance(other, Report)
            and self.name == other.name
            and self.input_parameters == other.input_parameters
            and self.output_parameters == other.output_parameters
        )


class ReportResult(Iterator):
    """DB report result.

    This object is an iterator. You can only iterate over it once.

    Raises:
        AmbiguousReportError: In case of ambiguous header (column names are not distinct).
    """

    def __init__(self, stream: Iterable):
        self._stream = stream
        self._generator_iterator = self._generator()

    def __next__(self) -> Dict[str, Any]:
        return next(self._generator_iterator)

    def _generator(self):
        """Provide generator method for report result iterator."""
        header_names = []  # type: List[str]
        for chunk in self._stream:
            if not header_names:
                header_names = [param['name'] for param in chunk['header']]
                if len(header_names) != len(set(header_names)):
                    raise AmbiguousColumnNames

            yield from (dict(zip(header_names, json.loads(row))) for row in chunk['rows'])

==> fred-reporter-0.3.2.tar.gz: fred-reporter-0.3.2/reporter/report.py <==
"""Report related data structures."""
import itertools
import json
from decimal import Decimal
from typing import Any, Dict, Generator, Iterable, Iterator, List, Sequence


class AmbiguousColumnNames(Exception):
    """Error thrown when report result doesn't have unique column names."""


class Param:
    """DB report parameter."""

    def __init__(self, name: str, type: str):
        self.name = name
        self.type = type

    def to_dict(self) -> Dict[str, str]:
        """Return param dict."""
        return {'name': self.name, 'type': self.type}

    def __repr__(self) -> str:
        return "Param<{}: {}>".format(self.name, self.type)

    def __eq__(self, other: Any) -> bool:
        return isinstance(other, Param) and self.name == other.name and self.type == other.type


class Report:
    """DB report."""

    def __init__(self, name: str, input_parameters: Sequence[Param], output_parameters: Sequence[Param]):
        self.name = name
        self.input_parameters = input_parameters
        self.output_parameters = output_parameters

    def to_dict(self) -> Dict[str, Any]:
        """Return report dict."""
        return {
            'name': self.name,
            'input_parameters': [param.to_dict() for param in self.input_parameters],
            'output_parameters': [param.to_dict() for param in self.output_parameters],
        }

    @property
    def is_unambiguous(self) -> bool:
        """Return whether all input param names are distinct and all output param names are distinct."""
        input_names = [param.name for param in self.input_parameters]
        output_names = [param.name for param in self.output_parameters]
        return len(input_names) == len(set(input_names)) and len(output_names) == len(set(output_names))

    def __eq__(self, other: Any) -> bool:
        return (
            isinstance(other, Report)
            and self.name == other.name
            and self.input_parameters == other.input_parameters
            and self.output_parameters == other.output_parameters
        )


class ReportResult(Iterator):
    """DB report result.

    This object is an iterator. You can only iterate over it once.
    JSON float is converted to Decimal.

    Raises:
        AmbiguousColumnNames: In case of ambiguous header (column names are not distinct).
    """

    def __init__(self, stream: Iterable):
        self._stream = stream
        generator = self._generator()
        try:
            first_row = next(generator)  # This can raise AmbiguousColumnNames
        except StopIteration:
            self._iterator: Iterator = generator  # Input stream is empty, we return an empty iterator
        else:
            self._iterator = itertools.chain([first_row], generator)

    def __next__(self) -> Dict[str, Any]:
        return next(self._iterator)

    def _generator(self) -> Generator[Dict[str, Any], None, None]:
        """Provide generator method for report result iterator."""
        header_names: List[str] = []
        for chunk in self._stream:
            if not header_names:
                header_names = [param['name'] for param in chunk['header']]
                if len(header_names) != len(set(header_names)):
                    raise AmbiguousColumnNames

            yield from (dict(zip(header_names, json.loads(row, parse_float=Decimal))) for row in chunk['rows'])
